'' This program requires a PropCAM-DB (#28320) plugged into a Propeller Backpack (#28327).
'' The Backpack's VID jumper should be installed, and the OVL jumper should be removed.
'' Video output is via the main (right) channel of the 2.5mm phone jack.

CON

  _clkmode      = xtal1 + pll8x
  _xinfreq      = 10_000_000

'' Constants for grayscale display.

  BASEPIN       = 12   'Base pin for video output.
  GRAYPIN       = 15   'Pin to use for DUTY-mode gray levels.
  WIDTH         = 128  'WIDTH of grayscale area in pixels. (WIDTH & 7 == 0 AND WIDTH => 128)
  HEIGHT        = 96   'HEIGHT of grayscale area in pixels. (HEIGHT => 96)

'' Pin definitions for Propeller Backpack. Can be changed for other Propeller modules.

  SCL           = 4
  SDA           = 5
  VSYNC         = 6
  MCLK          = 7
  DATA          = 0
  HSYNC         = 3

  MAX_COINS     = 16

DAT

  my_pins               byte    SCL, SDA, VSYNC, MCLK, DATA, HSYNC
  sizes                 byte    78, 64, 92
  values                byte     5, 10, 25

VAR

  byte sync
  long coins[MAX_COINS + 1], perims[MAX_COINS]

OBJ

  txt : "tv_wtext"              'Windowed version of tv_text.
  cam : "PropCAM_parallel4"     '4-bit-per-pixel PropCAM driver.
  ovl : "tv_gray4_overlay"      'Grayscale overlay for NTSC output.
  img : "ImageProcess4"         '4-bit-per-pixel image processor.
  pst : "Parallax Serial Terminal"
 
PUB start | cents, ncoins, ix, iy, perim, bbx, i, j, limit, best, diff

  dira[17]~~
  dira[16]~~
  pst.start(9600)
  txt.start(BASEPIN)
  gray_buf_addr := @gray_buf
  sync_addr := @sync
  ovl.start(BASEPIN, GRAYPIN, @gray_buf, 95, 28, 2, @sync)
  cam.start(cam#SYNCED, @my_pins, @gray_buf_addr)
  img.start(@gray_buf)
  txt.str(string(txt#DEFWIN, 1, 2, 1, 35, 12))
  txt.str(string(txt#DEFWIN, 2, 35, 1, 5, 4, txt#USEWIN, 1, txt#USEWIN, txt#ZERO))
  txt.str(string(txt#HOME, txt#SELCLR, 12, "PropCAM Coin-counting Demo", txt#USEWIN, 2))
  repeat
    cam.acquire(cam#GRAB)
    ncoins~
    repeat ix from 8 to 120 step 8
      repeat iy from 8 to 88 step 8
        perim := img.outline(ix, iy, 0, 0, 127, 95, img#UNDEF, %0000_1111_1111_1111, @stats)
        if (perim)
          bbx := bbx0 << 24 | bby0 << 16 | bbx1 << 8 | bby1
          limit := ncoins
          repeat i from 0 to limit
            if (i == ncoins)
              perims[ncoins] := perim                  
              coins[ncoins++] := bbx
              ncoins <#= constant(MAX_COINS)
            elseif (coins[i] == bbx)
              quit

    txt.str(string("                    "))
    cents~
    repeat i from 0 to ncoins - 1
      bbx := coins[i]
      bby1 := bbx & $ff
      bbx >>= 8
      bbx1 := bbx & $ff
      bbx >>= 8
      bby0 := bbx & $ff
      bbx >>= 8
      bbx0 := bbx & $ff
      bbx >>= 8
      img.box(bbx0, bby0, bbx1, bby1, 0,0,0)
      pst.dec(perims[i])
      pst.char(13)
      diff := $7fff_ffff
      repeat j from 0 to 3
        if (||(perims[i] - sizes[j]) < diff)
          diff := ||(perims[i] - sizes[j])
          best := j
      cents += values[best]    
    txt.str(string(txt#HOME, txt#SELCLR, 14, "Coins="))
    txt.dec(ncoins)
    txt.str(string(txt#CLREOL, txt#CR, "Cents="))
    txt.dec(cents)
    txt.str(string(txt#CLREOL))
    pst.char(13)


DAT

gray_buf      long      cam#PIXELS_PER_ROW << 16 | cam#ROWS_PER_FRAME, 0[cam#LONGS_PER_ROW * cam#ROWS_PER_FRAME]

gray_buf_addr word      0-0                     '@gray_buf gets plugged in here.
left          word      0                       'Lefmost pixel in gray_buf for capture.
top           word      0                       'Topmost oixel in gray_buf for capture.
exp_type      word      cam#AUTOBOTH            'Set exposure mode to automatically adjust gain and exposure time.
sync_addr     word      0-0                     '@sync gets plugged in here.
exp_time      word      20                      'Start with minimum exposure time.
gain          word      $0f                     'Start with medium gain.
target        word      1300                     'Autoexposure target for avg_pix.
time_interval word      -30                     'Initial exporsure time = 1/30 sec.
avg_pix       word      0-0                     'Average pixel value from last capture * 100.

stats
histogram     word      0[16]
bbx0          word      0
bby0          word      0
bbx1          word      0
bby1          word      0
ctrx          word      0
ctry          word      0